from .schemas import Location, User
from .exceptions import DecodeException
from typing import Union
from json import loads, dumps


_DATA_TYPES = {'Location': Location, 'User': User}
_EXTRA_FIELD = '_d_type'


def encode_data(data: Union[Location, User]):
    d = data.dict()
    d[_EXTRA_FIELD] = data.__class__.__name__
    return dumps(d)


def decode_data(data: bytes):
    obj = loads(data)
    d_type = obj[_EXTRA_FIELD]
    del obj[_EXTRA_FIELD]

    if d_type in _DATA_TYPES:
        return (d_type, _DATA_TYPES[d_type](**obj))

    raise DecodeException(f"cant parse type '{d_type}' in '{obj}")



from setuptools import find_packages, setup


setup(
    name='fldataparser',
    version='0.0.3',
    description='parser tool for object send through rabbitmq in FLExplorer',
    packages=find_packages()
)



from pydantic import BaseModel, validator
from typing import Optional


def _normalize(val: str) -> str:
    if val is None:
        return None
    return val.lower()


class Location(BaseModel):
    name: str
    parent_name: Optional[str] = None

    _name_val = validator('name', allow_reuse=True)(_normalize)
    _parent_name_validator = validator(
        'parent_name', allow_reuse=True
        )(_normalize)


class User(BaseModel):
    fl_id: int
    name: str
    birth_year: int
    gender: Optional[str] = None
    role: Optional[str] = None
    pics: int
    vids: int
    profile_pic_url: Optional[str] = None
    location: Location

    _gender_val = validator('gender', allow_reuse=True)(_normalize)
    _role_val = validator('role', allow_reuse=True)(_normalize)
